<?php
if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class All_Offers extends WP_List_Table {

    /** Class constructor */
    public function __construct()
    {
        parent::__construct([
            'singular' => __('Offer', 'sp'), //singular name of the listed records
            'plural' => __('Offers', 'sp'), //plural name of the listed records
            'ajax' => false //should this table support ajax?
        ]);

    }

    /**
     * Retrieve All Offers from the database
     *
     * @param int $per_page
     * @param int $page_number
     *
     * @return mixed
     */
    public static function get_offers( $per_page = 15, $page_number = 1 ) {

        global $wpdb;

        $sql = "SELECT  p.product_id, wp.post_title as product_name, COUNT(p.product_id) as orders, SUM(p.quantity) as quantity FROM {$wpdb->prefix}dbargain_reports AS p JOIN {$wpdb->prefix}posts AS wp ON p.product_id = wp.ID GROUP BY p.product_id";

        if ( ! empty( $_REQUEST['orderby'] ) ) {
            $sql .= ' ORDER BY ' . esc_sql( $_REQUEST['orderby'] );
            $sql .= ! empty( $_REQUEST['order'] ) ? ' ' . esc_sql( $_REQUEST['order'] ) : ' ASC';
        }

        $sql .= " LIMIT $per_page";

        $sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;

        $result = $wpdb->get_results( $sql, 'ARRAY_A' );

        $dex=0;
        foreach($result as $p)
        {
            $result[$dex]['customers'] = $wpdb->get_var("select count(distinct user_id) from {$wpdb->prefix}dbargain_reports where product_id = '".$p['product_id']."'");
            $dex++;
        }

        return $result;
    }

    /**
     * Returns the count of reports in the database.
     *
     * @return null|string
     */
    public static function record_count() {
        global $wpdb;

        $sql = "SELECT COUNT(DISTINCT product_id) FROM {$wpdb->prefix}dbargain_reports";

        return $wpdb->get_var( $sql );
    }

    /** Text displayed when no data is available */
    public function no_items() {
        _e( 'No data.', 'sp' );
    }

    /**
     * Method for Product ID column
     *
     * @param array $item an array of DB data
     *
     * @return string
     */
    /*function column_product_id( $item ) {

        // create a nonce
        //$delete_nonce = wp_create_nonce( 'sp_delete_customer' );

        $title = '<strong>' . $item['product_id']. '</strong>';

        $actions = [
            'edit' => sprintf( '<a href="?page=%s">Edit</a>', esc_attr( 'dbargain' ) )
        ];

        return $title . $this->row_actions( $actions );
    }*/

    /**
     * Method for view details
     *
     * @param array $item an array of DB data
     *
     * @return string
     */
    function column_details( $item ) {

        $title = '<a href="javascript:;" class="show_detail populate_detail" rel="'.$item['product_id'].'">Show Details</a>';

        return $title;
    }

    /**
     * Render a column when no column specific method exists.
     *
     * @param array $item
     * @param string $column_name
     *
     * @return mixed
     */
    public function column_default( $item, $column_name ) {
        return $item[ $column_name ];
    }

    /**
     * Render the bulk update checkbox
     *
     * @param array $item
     *
     * @return string
     *//*
    function column_cb( $item ) {
        return sprintf(
            '<input type="checkbox" name="bulk-action[]" value="%s" />', $item['product_id']
        );
    }*/

    /**
     * Associative array of columns
     *
     * @return array
     */
    function get_columns() {
        $columns = [
        //    'cb' => '<input type="checkbox" />',
            'product_id' => __( 'Product ID', 'sp' ),
            'product_name' => __( 'Product Name', 'sp' ),
            'customers' => __( 'Total Buyers', 'sp' ),
            'orders' => __('Total No. of Orders','sp'),
            'quantity' => __( 'Total Quantity Sold', 'sp' ),
            'details' => ''
        ];

        return $columns;
    }

    /**
     * Columns to make sortable.
     *
     * @return array
     */
    public function get_sortable_columns() {
        $sortable_columns = array(
            'customers' => array( 'customers', true ),
            'orders' => array( 'orders', true ),
            'quantity' => array( 'quantity', true )
        );

        return $sortable_columns;
    }

    /**
     * Returns an associative array containing the bulk action
     *
     * @return array
     *//*
    public function get_bulk_actions() {
        $actions = [
            'bulk-export' => 'Export'
        ];

        return $actions;
    }*/

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Handles data query and filter, sorting, and pagination.
     */
    public function prepare_items() {

        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        /** Process bulk action */
        //$this->process_bulk_action();

        $perPage = 15;
        $currentPage = $this->get_pagenum();
        $totalItems = self::record_count();

        $data = self::get_offers( $perPage, $currentPage );

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        //$data = array_slice($data,(($currentPage-1)*$perPage),$perPage);

        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /**
     * Process bulk update request
     *//*
    public function process_bulk_action() {

        global $wpdb;

        if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-export' )
            || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-export' )
        ) {

        }

        return;
    }*/
}