    <?php

    class DBargain {

        function __construct() {
            return $this;
        }

        public static function activate()
        {
            flush_rewrite_rules();

            global $pt_db_version, $wpdb;
            $pt_db_version = '1.0';

            $dbargain_table_name = $wpdb->prefix . 'dbargain_reports';
            $session_table_name = $wpdb->prefix . 'dbargain_session';
            $response_table_name = $wpdb->prefix . 'dbargain_responses';

            $charset_collate = $wpdb->get_charset_collate();

            $sql_dbargain = "CREATE TABLE IF NOT EXISTS $dbargain_table_name (
                                ID mediumint(9) NOT NULL AUTO_INCREMENT,
                                session_id varchar(128) NULL,
                                user_id mediumint(9) NULL,
                                product_id mediumint(9) NOT NULL,
                                quantity mediumint(9) NOT NULL,
                                order_price float(5,2) NULL,
                                date_created datetime NOT NULL,
                                PRIMARY KEY (ID)
                            ) $charset_collate;";

            $sql_session = "CREATE TABLE IF NOT EXISTS $session_table_name (
                                ID mediumint(9) NOT NULL AUTO_INCREMENT,
                                session_id VARCHAR(128) NOT NULL,
                                product_id mediumint(9) NOT NULL,
                                offer float(6,2) NULL,
                                message text NULL,
                                status tinyint(4) NOT NULL,
                                date_created datetime NOT NULL,
                                PRIMARY KEY (ID)
                            ) $charset_collate;";

            $sql_responses = "CREATE TABLE IF NOT EXISTS $response_table_name (
                                ID mediumint(9) NOT NULL AUTO_INCREMENT,
                                percentage_difference mediumint(4) NULL,
                                `condition` VARCHAR(128) NULL,
                                message text NOT NULL,
                                PRIMARY KEY (ID)
                            ) $charset_collate;";

            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            //echo $sql_project;die;
            dbDelta( $sql_dbargain );
            dbDelta( $sql_session );
            dbDelta( $sql_responses );
            
            $wpdb->query("insert  into {$wpdb->prefix}dbargain_responses(`ID`,`percentage_difference`,`condition`,`message`) values (1,10,'less','Your offer is very good but you are just slightly under our accepted limit. Please try again'),(2,20,'less','You are getting closer but still you need to give a better offer'),(3,30,'less','We don\'t wish to be rude but this product is worth a bit more than what you are offering'),(4,40,'less','The offer doesn\'t match with the worth of product. Please improve'),(5,50,'less','You need to give a better offer than this'),(6,60,'less','Please make a serious offer considering the worth of the product.'),(7,NULL,'more','Thankyou for your enthusiasm but the offer you are making, is above the product price itself. You can buy the product on original price'),(8,NULL,'success','Success!! Thankyou for the great offer. You can buy the product on your offered price'),(9,NULL,'warning','You have exhausted all your attempts. Considering your interest and effort, we are going to grant you one last chance. Please give us your best offer this time.'),(10,NULL,'welcome','Welcome, Thankyou for taking interest in this product. Please give us your best offer.'),(11,NULL,'failure','We are sorry but you have exhausted all your chances. Considering your interest and effort, we can let you buy this product on a discounted price still. Our final price for you is: ');
    ");

            add_option( 'pt_db_version', $pt_db_version );
        }

        public static function deactivate()
        {
            flush_rewrite_rules();
        }

        public static function uninstall()
        {
            global $wpdb;

            $wpdb->query( "DROP TABLE IF EXISTS ".$wpdb->prefix . 'dbargain_reports' );
            $wpdb->query( "DROP TABLE IF EXISTS ".$wpdb->prefix . 'dbargain_session' );
            $wpdb->query( "DROP TABLE IF EXISTS ".$wpdb->prefix . 'dbargain_responses' );

            delete_option("pt_db_version");

        }

        public static function register() {

            //Actions

            /* Hook into the 'init' action so that the function
            * Containing our post type registration is not
            * unnecessarily executed.
            */

            add_action('admin_menu', [self::class, 'add_admin_pages']);
            // Register javascript
            add_action('admin_enqueue_scripts', [self::class, 'enqueue_admin_js'] );
            add_action('wp_enqueue_scripts', [self::class, 'enqueue_frontend_js'] );

            add_action( 'wp_ajax_nopriv_fetch_details', [self::class, 'fetch_details'] );
            add_action( 'wp_ajax_fetch_details', [self::class, 'fetch_details'] );

            // The code for displaying WooCommerce Product Custom Fields
            add_action( 'woocommerce_product_options_general_product_data', [self::class, 'dbargain_fields'] );
            // Following code Saves  WooCommerce Product Custom Fields
            add_action( 'woocommerce_process_product_meta', [self::class, 'dbargain_fields_save'] );

            //Code for overriding cart prices
            add_action('woocommerce_before_calculate_totals', [self::class, 'update_cart_prices'],10,1 );
            //add_filter('woocommerce_cart_item_price', [self::class, 'update_price_mini_cart'],30,1 );
        // add_filter('woocommerce_get_cart_contents', [self::class, 'update_cart_prices'],30,1 );

            //Code to unset session data after checkout
            add_action('woocommerce_thankyou', [self::class, 'unset_session'], 10, 1);

            //add_action('wp_footer', [self::class, 'render_dbargain_window']);
        }

        public static function unset_session($order_id)
        {
            if(isset($_SESSION['dbargain'])) {
                unset($_SESSION['dbargain']);
                session_regenerate_id();
            }
        }

        public static function update_cart_prices($cart)
        {
            // This is necessary for WC 3.0+
            if ( is_admin() && ! defined( 'DOING_AJAX' ) )
                return;

            // Avoiding hook repetition (when using price calculations for example | optional)
            if ( did_action( 'woocommerce_before_calculate_totals' ) >= 2 )
                return;

            // Loop through cart items
            foreach ( WC()->cart->get_cart() as $cart_item ) {
                $id = $cart_item['data']->get_id();

                if(isset($_SESSION['dbargain'][$id]['dbargain_price']) && !empty($_SESSION['dbargain'][$id]['dbargain_price']))
                $cart_item['data']->set_price( $_SESSION['dbargain'][$id]['dbargain_price'] );
            }
        }

        public static function update_price_mini_cart($price_html, $cart_item, $cart_item_key)
        {
            $id = $cart_item['data']->get_id();

            if(isset($_SESSION['dbargain'][$id]['dbargain_price']) && !empty($_SESSION['dbargain'][$id]['dbargain_price']))
            {
                $price = ['price'=>$_SESSION['dbargain'][$id]['dbargain_price']];
                if ( WC()->cart->display_prices_including_tax() ) {
                    $product_price = wc_get_price_including_tax( $cart_item['data'], $price );
                } else {
                    $product_price = wc_get_price_excluding_tax( $cart_item['data'], $price );
                }
                return wc_price( $product_price );
            }

            return $price_html;
        }

        //start session
        public static function start_session()
        {
            session_start();
            if(!isset($_SESSION['dbargain']))
                $_SESSION['dbargain'] = [];
        }

        //Ajax function to get offer details
        public static function fetch_details()
        {
            global $wpdb;

            $product = wc_get_product( $_POST['id'] );

            $data = $wpdb->get_results("select distinct user_id from {$wpdb->prefix}dbargain_reports where product_id = '".$_POST['id']."'",'ARRAY_A');

            $image = wp_get_attachment_image_src( get_post_thumbnail_id($_POST['id']), 'product');

            $html = "
                    <table class='wp-list-table widefat fixed table-view-list offers' border='0' width='100%'>
                        <tr>
                            <td width='10%'>
                                <img src='".$image[0]."' width='100px'>
                            </td>
                            <td width='11%'>
                                <p>#".$_POST['id']."</p>
                                <p><h2>".$product->get_title()."</h2></p>
                                <p>".$product->get_short_description()."</p>
                                <p>".wc_price($product->get_regular_price())."</p>
                            </td>
                            <td width='78%'>
                                <table class='wp-list-table widefat fixed table-view-list offers' border='0' width='100%'>
                                    <tr>
                                        <td style='background-color:#efefef'><b>Customer Name</b></td>
                                        <td style='background-color:#efefef'><b>Bargain Price</b></td>
                                        <td style='background-color:#efefef'><b>Past Orders</b></td>
                                        <td style='background-color:#efefef'><b>Email</b></td>
                                        <td style='background-color:#efefef'><b>Phone</b></td>
                                        <td style='background-color:#efefef'><b>Status</b></td>
                                        <td style='background-color:#efefef'><b>Chat History</b></td>
                                    </tr>";
            foreach($data as $d) {
                $user = get_user_by('id',$d['user_id']);
                $session = $wpdb->get_row("select session_id, order_price from {$wpdb->prefix}dbargain_reports where user_id = '{$d["user_id"]}' and product_id = '{$_POST["id"]}' order by ID DESC  limit 0,1",ARRAY_A);
                $html .= "
                                    <tr>
                                        <td>".$user->display_name."</td>
                                        <td>".((isset($session['order_price']) && !empty($session['order_price']))?$session['order_price']:'')."</td>
                                        <td>".$wpdb->get_var("select count(ID) from {$wpdb->prefix}dbargain_reports where user_id = '{$d["user_id"]}' and product_id = '{$_POST["id"]}'")."</td>
                                        <td>".$user->user_email."</td>
                                        <td>".get_user_meta($d['user_id'],'user_phone',true)."</td>
                                        <td>".((isset($session['order_price']) && !empty($session['order_price']))?'Sold':'Failed')."</td>
                                        <td><a href='?page=chat&sid=".$session['session_id']."'>Show Chat</a></td>
                                    </tr>";
            }

                $html .= "
                                </table>
                            </td>
                        </tr>
                        <tr>
                            <td colspan='3' style='text-align:right'><a href='javascript:;' onclick='hide_detail(".$_POST['id'].")' class='hide_detail' rel='".$_POST['id']."'>Hide Details</a></td>
                        </tr>
                    </table>
                    ";

            wp_send_json_success(['id'=>$_POST['id'],'data'=>$html]);
        }

        /**
         * Function that will add javascript file for Color Piker.
         */
        public static function enqueue_admin_js() {
    // Css rules for Color Picker
            wp_enqueue_style( 'wp-color-picker' );
            // Make sure to add the wp-color-picker dependecy to js file
            wp_enqueue_script( 'dbargain_admin_js', plugins_url( 'assets/javascript.js', __FILE__ ), array( 'jquery', 'wp-color-picker' ), '', true  );

            wp_enqueue_style( 'jquery-ui' );
            wp_enqueue_script( 'jquery-ui-datepicker' );

            
        }

        /**
         * Function that will add javascript file for DBargain popup.
         */
        public static function enqueue_frontend_js() {
            wp_enqueue_script( 'dbargain_frontend_js', plugins_url( 'assets/javascript_frontend.js', __FILE__ ),array('jquery'),'',true );
        }

        //Function to create custom fields on product edit page
        public static function dbargain_fields()
        {
            echo '<div class="product_custom_field">';
            // DBargain Price Threshold Field
            woocommerce_wp_text_input(
                array(
                    'id' => '_dbargain_price_threshold',
                    'placeholder' => 'Bargain Price Threshold',
                    'label' => __('Bargain Price Threshold', 'woocommerce'),
                    'desc_tip' => 'true'
                )
            );

            // DBargain Date Start Field
            woocommerce_wp_text_input(
                array(
                    'id' => '_dbargain_date_start',
                    'placeholder' => 'Bargain Start Date',
                    'class' =>'datepicker',
                    'label' => __('Bargain Start Date', 'woocommerce'),
                    'desc_tip' => 'true'
                )
            );

            // DBargain Date End Field
            woocommerce_wp_text_input(
                array(
                    'id' => '_dbargain_date_end',
                    'placeholder' => 'Bargain End Date',
                    'class' => 'datepicker',
                    'label' => __('Bargain End Date', 'woocommerce'),
                    'desc_tip' => 'true'
                )
            );

            echo '</div>';
        }

        //Function to save custom fields from product edit page
        public static function dbargain_fields_save()
        {
            global $post_id;

            // DBargain Price Threshold Field
            $price_threshold = $_POST['_dbargain_price_threshold'];
            if (!empty($price_threshold))
                update_post_meta($post_id, '_dbargain_price_threshold', esc_attr($price_threshold));

            // DBargain Price Threshold Field
            $start_date = $_POST['_dbargain_date_start'];
            if (!empty($start_date))
                update_post_meta($post_id, '_dbargain_date_start', esc_attr($start_date));

            // DBargain Price Threshold Field
            $end_date = $_POST['_dbargain_date_end'];
            if (!empty($end_date))
                update_post_meta($post_id, '_dbargain_date_end', esc_attr($end_date));
        }

        //Function to create admin menu
        public static function add_admin_pages() {
            add_menu_page('D-Bargain', 'D-Bargain', 'manage_options', 'dbargain', [self::class, 'all_offers'], 'dashicons-admin-settings', 100);
            add_submenu_page('dbargain', 'All Offers', 'All Offers', 'manage_options', 'dbargain', [self::class, 'all_offers']);
            add_submenu_page('dbargain', 'Settings', 'Settings', 'manage_options', 'dbargain_settings', [self::class, 'settings']);
            add_submenu_page('dbargain', 'Reports', 'Reports', 'manage_options', 'reports', [self::class, 'reports']);
            add_submenu_page('', 'Session Chat History', 'Session Chat History', 'manage_options', 'chat', [self::class, 'chat']);
        }

        //Function to render all offers admin menu page template
        public static function all_offers() {
            require_once DBARGAIN_PLUGIN_PATH . 'templates/all-offers.php';
        }

        //Function to render settings admin menu page template
        public static function settings() {
            require_once DBARGAIN_PLUGIN_PATH . 'templates/settings.php';
        }

        //Function to render reports admin menu page template
        public static function reports() {
            require_once DBARGAIN_PLUGIN_PATH . 'templates/reports.php';
        }

        //Function to render chat history admin menu page template
        public static function chat() {
            require_once DBARGAIN_PLUGIN_PATH . 'templates/chat.php';
        }

        public static function chat_message_response($agent_name,$msg,$offer)
        {
            $res = '<div class="chat-message clearfix">';
                                
            if($offer == null){

                $res .= '<img src="http://gravatar.com/avatar/2c0ad52fc5943b78d6abe069cc08f320?s=32" alt="" width="32" height="32">';

            }
            
            $res .= '
                    <div class="chat-message-content clearfix">
                        
                        <!--<span class="chat-time">13:37</span>-->

                    ';

                if($offer == null){

                    $res .= '<h5>'.$agent_name.'</h5>';        
                }
                
                if($offer == null){

                    $res .= '<p>'.$msg.'</p>';   
                }else{

                    $res .= '<p style="text-align: right;">'.$msg.'</p>';   
                }
            
                $res .= '
            
                    </div> <!-- end chat-message-content -->
            
                </div> <!-- end chat-message -->
            
                <hr>';
    
        return $res;
    
        }
    }

    //Message formate


    DBargain::register();